@@ -20,8 +20,8 @@ class CaretChange(Base.AccessEngineEvent):
   Event that fires when the caret moves in an accessible.
   
   This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor.EventMonitor} using the L{Constants.registerEventType} function
-  when this module is first imported. The L{AEMonitor.EventMonitor} can use this
+  an L{AEMonitor} using the L{Constants.registerEventType} function
+  when this module is first imported. The L{AEMonitor} can use this
   information to build its menus.
   
   @ivar text: The text inserted, deleted or the line of the caret
