@@ -1,12 +1,12 @@
 '''
 Defines a function that allows L{AEEvent} classes to register their names to
 suggest all possible event names and defaults to buffer to an
-L{AEMonitor.EventMonitor}s. Also defines priority constants that determine when
+L{AEMonitor}s. Also defines priority constants that determine when
 L{AEEvent}s execute.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made 
@@ -15,9 +15,11 @@ this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
 '''
 # layer constants, with guaranteed order from most specific to least specific
-FOCUS_LAYER = 0
-TIER_LAYER = 1
-BACKGROUND_LAYER = 2
+LAYER_FOCUS = 0
+LAYER_TIER = 1
+LAYER_BACKGROUND = 2
+LAYERS_ALL = [LAYER_FOCUS, LAYER_TIER, LAYER_BACKGROUND]
+LAYER_NAMES = ['focus', 'tier', 'background']
 
 # priority constants
 NORMAL = 0
@@ -32,7 +34,7 @@ all_types = []
 def registerEventType(name, default):
   '''
   Called once by every L{AEEvent} class to register itself for buffering and 
-  filtering by an L{AEMonitor.EventMonitor}.
+  filtering by an L{AEMonitor}.
   
   @param name: Name of the L{AEEvent}
   @type name: string
@@ -43,4 +45,4 @@ def registerEventType(name, default):
   global all_types
   all_types.append(name)
   if default:
-    default_types.append(name)
\ No newline at end of file
+    default_types.append(name)
