@@ -30,4 +30,4 @@ class InitError(AEInputError):
   '''
   def __init__(self, *args, **kwargs):
     AEOutputError.__init__(self,
-      _('The AEInput device could not be initialized'), *args, **kwargs)
\ No newline at end of file
+      _('could not initialize input device'), *args, **kwargs)
\ No newline at end of file
