@@ -13,4 +13,4 @@ this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
 '''
 from Base import AEState
-from Setting import Setting
\ No newline at end of file
+from Setting import *
\ No newline at end of file
