@@ -79,6 +79,69 @@ class TableAccInfoAdapter(ContainerAccInfoAdapter):
     tab = Interfaces.ITable(self.accessible)
     return tab.getColumnAtIndex(self.item_offset)
 
+  @pyLinAcc.errorToLookupError
+  def getAccRowColIndex(self, row, col):
+    '''
+    Gets the 1D index of the cell at the given 2D row and column.
+    
+    @param row: Row index
+    @type row: integer
+    @param col: Column index
+    @type col: integer
+    @return: 1D index into the table
+    @rtype: integer
+    @raise IndexError: When the row/column offsets are invalid
+    @raise LookupError: When the table is no longer valid
+    '''
+    tab = Interfaces.ITable(self.accessible)
+    i = tab.getIndexAt(row, col)
+    if i < 0:
+      raise IndexError
+    return i
+
+  @pyLinAcc.errorToLookupError 
+  def getAccRowHeader(self):
+    '''
+    Gets the text description of a row in a table.
+
+    @return: The descriptive text.
+    @rtype: string
+    @raise LookupError: When the table or item is no longer valid
+    '''
+    tab = Interfaces.ITable(self.accessible)
+    if self.item_offset != None:
+      row = tab.getRowAtIndex(self.item_offset)
+      return tab.getRowDescription(row)
+    return None
+    
+  @pyLinAcc.errorToLookupError 
+  def getAccColumnHeader(self):
+    '''
+    Gets the text description of a column in a table.
+
+    @return: The descriptive text.
+    @rtype: string
+    @raise LookupError: When the table or item is no longer valid
+    '''
+    tab = Interfaces.ITable(self.accessible)
+    if self.item_offset != None:
+      col = tab.getColumnAtIndex(self.item_offset)
+      return tab.getColumnDescription(col)
+    return None
+
+  @pyLinAcc.errorToLookupError
+  def getAccTableExtents(self):
+    '''
+    Returns the number of rows and columns in the table.
+
+    @return: Count of rows and columns
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the table is no longer valid
+    '''
+    tab = Interfaces.ITable(self.accessible)
+    return (tab.nRows, tab.nColumns)
+
+  
 class TableNavAdapter(DefaultNavAdapter):
   '''
   Overrides L{DefaultNavAdapter} to provide navigation over table cells as 
@@ -107,7 +170,11 @@ class TableNavAdapter(DefaultNavAdapter):
       for this class, False otherwise
     @rtype: boolean
     '''
-    return Interfaces.ITable(subject.accessible)
+    acc = subject.accessible
+    ss = acc.getState()
+    if not ss.contains(Constants.STATE_MANAGES_DESCENDANTS):
+      return False
+    return Interfaces.ITable(acc)
   
   @pyLinAcc.errorToLookupError
   def _getVisibleItemExtents(self):
@@ -125,10 +192,16 @@ class TableNavAdapter(DefaultNavAdapter):
     e = comp.getExtents(Constants.WINDOW_COORDS)
     # get the first item
     x, y = e.x+FUDGE_PX, e.y+FUDGE_PX
-    first = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
+    try:
+      first = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
+    except TypeError:
+      first = None
     # get the last item
     x, y = e.x+e.width-FUDGE_PX, e.y+e.height-FUDGE_PX
-    last = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
+    try:
+      last = comp.getAccessibleAtPoint(x, y, Constants.WINDOW_COORDS)
+    except TypeError:
+      last = None
     # compute indices
     if first:
       i = first.getIndexInParent()
@@ -274,7 +347,7 @@ class TableNavAdapter(DefaultNavAdapter):
     child = acc.getChildAtIndex(0)
     if IAccessibleInfo(POR(child)).isAccVisible() or not only_visible:
       return POR(acc, 0, 0)
-    # use coords to get the last visible item
+    # use coords to get the first visible item
     i, j = self._getVisibleItemExtents()
     return POR(acc, i, 0)
   
@@ -376,18 +449,12 @@ class DegenerateTableEventHandlerAdapter(DefaultEventHandlerAdapter):
     #   The Selection interface has a list of one or more selected items.
     #   selection.nSelectedChildren indicates how many there are and 
     #   selection.getSelectedChild(index) accesses one of them.
-    
-    # determine if there is a Text interface
-    try:
-      text = Interfaces.IText(accessible)
-      acc_child_text = text.getText(0, text.characterCount)   
-    except NotImplementedError:      
-      acc_child_text = accessible.name
+
 
     # create a POR, pass it and the item text at the POR to the Tier
     por = POR(self.subject, item_offset, 0)
-    return SelectorChange(por, unicode(acc_child_text, 'utf-8'), 
-                          **kwargs)
+    acc_child_text = IAccessibleInfo(por).getAccItemText()
+    return SelectorChange(por, acc_child_text, **kwargs)
   
 class TableEventHandlerAdapter(DegenerateTableEventHandlerAdapter):
   '''  
@@ -481,4 +548,4 @@ class TableEventHandlerAdapter(DegenerateTableEventHandlerAdapter):
       return (focus_event,)
     item_offset = acc_child.getIndexInParent()
     # build and return a selector event
-    return focus_event, self._getSelectorEvent(acc_child, item_offset,**kwargs)
\ No newline at end of file
+    return focus_event, self._getSelectorEvent(acc_child, item_offset,**kwargs)
