@@ -6,7 +6,7 @@ are not imported.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made
@@ -17,8 +17,4 @@ U{http://www.opensource.org/licenses/cpl1.0.php}
 try:
   from ATSPI import *
 except ImportError, e:
-  print 'AT-SPI adapters unavailable:', e
-
-# register all Adapter subclasses
-import AccAdapt
-AccAdapt.registerAdapters()  
+  print 'AT-SPI adapters unavailable:', e
\ No newline at end of file
